(* ::Package:: *)

(* ::Title:: *)
(*Lee-Pomeranski*)


SetDirectory @ NotebookDirectory[];
Block[{Print=Nothing}, <<LiteRed`];
Get["../gkz_utils/gkz.m"]


(* ::Section:: *)
(*Kinematics*)


SetDim[d];
Declare[{k,p1,p2,p3,p4},Vector];
Declare[{s12,s13,s23,s14,s24,s34,P5}, Number];

sp[p1] = 0;
sp[p2] = 0;
sp[p3] = 0;
sp[p4] = 0;
(* sp[p5] = P5 *)

sp[p1,p2] = s12/2;
sp[p1,p3] = s13/2;
sp[p2,p3] = s23/2;
sp[p1,p4] = s14/2;
sp[p2,p4] = s24/2;

sp[p3,p4] = P5/2 - (sp[p1,p2] + sp[p1,p3] + sp[p1,p4] + sp[p2,p3] + sp[p2,p4]);


(* ::Text:: *)
(*derivation of sp[p3,p4] using that (p1 + p2 + p3 + p4)^2 = (-p5)^2 = P5*)


(p@1 + p@2 + p@3 + p@4 )^2 - P5 // RightComposition[
	Expand,
	ReplaceAll[{
		p[i_]^2 :> 0 , 
		p[i_]p[j_] :> S[i,j]
	}],
	Solve[# == 0 , S[3,4]]&,
	First,
	Expand,
	Identity
]


dens = Table[k - Sum[p@i // ind2sym, {i,n}] // sp, {n,0,4}]


(* ::Section:: *)
(*Variables*)


zs = {z1, z2, z3, z4, z5}
s = {s12, s13, s23, s14, s24, P5}

scale = s[[1]]
scomp = Complement[s , {scale}]

zrep = Thread[# -> #/scale& @ zs]
srep = Table[scomp[[n]] -> (r@n // ind2sym) * scale, {n,Length@scomp}]


(* ::Section:: *)
(*Polynomial*)


pent = FeynParUFFunction[dens, {k}] @@ zs // RightComposition[
	Total,
	ReplaceAll @ zrep,
	Multiply[scale],
	ReplaceAll @ srep,
	Expand
];


gkzform[pent , zs];
%[[1]]
Print[""];
%%%[[2]] // Column 


(* ::Section:: *)
(*Simplex*)


myA = Amat[pent,zs];
myA // asirlist


\[Sigma] = Range[6]
myA[[All,\[Sigma]]] // Det


(* ::Section:: *)
(*IBPs*)


(* ::Text:: *)
(*takes 15 mins*)


(*Clear[pent]

Block[{Print = Nothing},
	SetOptions[FindSymmetries, EMs -> True];
	NewBasis[pent, dens, {k}];
	
	GenerateIBP[pent];
	AnalyzeSectors[pent];
	FindSymmetries[pent];
	SolvejSector[#, SR -> True]& /@ UniqueSectors[pent];
]*)


(*MIs[pent] *)


mis = {
j[pent,0,0,1,0,1],
j[pent,0,1,0,0,1],
j[pent,0,1,0,1,0],
j[pent,1,0,0,0,1],
j[pent,1,0,0,1,0],
j[pent,1,0,1,0,0],
j[pent,1,0,1,0,1],
j[pent,0,1,1,1,1],
j[pent,1,0,1,1,1],
j[pent,1,1,0,1,1],
j[pent,1,1,1,0,1],
j[pent,1,1,1,1,0],
j[pent,1,1,1,1,1]
};
