(* ::Package:: *)

(* ::Subtitle:: *)
(*Check Pfaffians against LiteRed*)


SetDirectory @ NotebookDirectory[];
Get["../gkz_utils/gkz.m"]
Block[{Print=Nothing}, <<LiteRed`];
Get["FiniteFlow.m"]


(* ::Section:: *)
(*IBPs*)


SetDim[d];
Declare[{k,p1,p2,p3,p4},Vector];
Declare[{s12,s13,s23,s14,s24,s34,P5}, Number];

sp[p1] = 0;
sp[p2] = 0;
sp[p3] = 0;
sp[p4] = 0;
(* sp[p5] = P5 *)

sp[p1,p2] = s12/2;
sp[p1,p3] = s13/2;
sp[p2,p3] = s23/2;
sp[p1,p4] = s14/2;
sp[p2,p4] = s24/2;

sp[p3,p4] = P5/2 - (sp[p1,p2] + sp[p1,p3] + sp[p1,p4] + sp[p2,p3] + sp[p2,p4]);


dens = (-1) * Table[k - Sum[p@i // ind2sym, {i,n}] // sp, {n,0,4}]


Clear[pent]

Block[{Print = Nothing},
	SetOptions[FindSymmetries, EMs -> True];
	NewBasis[pent, dens, {k}];
	
	GenerateIBP[pent];
	AnalyzeSectors[pent];
	FindSymmetries[pent];
	SolvejSector[#, SR -> True]& /@ UniqueSectors[pent];
]


(* ::Section:: *)
(*LP*)


(* ::Text:: *)
(*here x[i] denotes integration variables*)


poly = 
	x[1] + 
	x[2] + 
	x[3] + 
	x[4] + 
	x[5] +
	x[1] * x[3] + 
	(1 + y2 + y4) * x[1] * x[4] + 
	y1 * x[1] * x[5] + 
	y4 * x[2] * x[4] + 
	(-1 + y1 - y2 - y3) * x[2] * x[5] + 
	(-1 + y1 - y2 - y3 - y4 - y5) * x[3] * x[5]


(* ::Text:: *)
(*the Pfaffians are sadly written with xi representing zi, so 'x' in 'xtoy' refers to external variables*)


xtoy = {
	x7  -> 1 + y2 + y4,
	x8  -> y1,
	x9  -> y4,
	x10 -> -1 + y1 - y2 - y3,
	x11 -> -1 + y1 - y2 - y3 - y4 - y5
} // ReplaceAll


stoy = {
	P5 -> y1 s12,
	s13 -> y2 s12, 
	s14 -> y3 s12, 
	s23 -> y4 s12, 
	s24 -> y5 s12
} // ReplaceAll


(* ::Section:: *)
(*bases*)


(* ::Subsection:: *)
(*integrals*)


(* ::Text:: *)
(*notation: J[d0, nu__]*)


ints = {
	J[2,0,0,1,0,1],
	J[2,0,1,0,0,1],
	J[2,0,1,0,1,0],
	J[2,1,0,0,0,1],
	J[2,1,0,0,1,0],
	J[2,1,0,1,0,0],
	J[4,1,0,1,0,1],
	J[4,0,1,1,1,1],
	J[6,1,0,1,1,1],
	J[4,1,1,0,1,1],
	J[4,1,1,1,0,1],
	J[4,1,1,1,1,0],
	J[6,1,1,1,1,1]
};


(* ::Text:: *)
(*mixed dimensional basis*)


e246 = ints /. J[d0_,nu__] :> j[pent,nu]


(* ::Text:: *)
(*ints in dimensions 2 and 6*)


pos2 = Position[ints, J[2,__]] // Flatten
pos6 = Position[ints, J[6,__]] // Flatten


(* ::Subsection:: *)
(*dim rec*)


(* ::Text:: *)
(*2 to 4*)


dim2to4 = (-1) * RaisingDRR @@@ e246[[pos2]] // IBPReduce 


(* ::Text:: *)
(*6 to 4*)


dim6to4 = (-1) * LoweringDRR @@@ e246[[pos6]] // IBPReduce; 


(* ::Subsection:: *)
(*4d basis*)


(* ::Text:: *)
(*purely 4d basis*)


e4 = ReplacePart[
	e246, 
	Join[
		Thread[pos2 -> dim2to4], 
		Thread[pos6 -> dim6to4]
	]
];


(* ::Text:: *)
(*e4 = R \[Bullet] e246*)


R = CoefficientArrays[e4, e246][[2]] // Normal // FullSimplify;


(* ::Section:: *)
(*DEQ*)


(* ::Subsection:: *)
(*from Macaulay matrix (mm)*)


(* ::Text:: *)
(*Jacobians w.r.t y1 = p^2/s12*)


jacmm = D[xtoy[[1,All,2]],y1]


{p7, p8, p9, p10, p11} = Import["pfaffians_x7_x8_x9_x10_x11_basis_dims_2_4_6.m"] // xtoy // Factor;


deqmm = jacmm . {p7, p8, p9, p10, p11} // Factor;


deqmm // Variables


(* ::Subsection:: *)
(*from LiteRed (lr)*)


deqP5 = e246 // RightComposition[
	Dinv[#,P5]&,
	IBPReduce,
	CoefficientArrays[#,e246]&,
	Part[#,2]&,
	Normal,
	Factor,
	Identity
];


jaclr = s12


prefacs = Import["prefactors_basis_dims_2_4_6.m"] /. s12 -> -s12;


(* ::Text:: *)
(*gauge*)


G = prefacs . R;


deqlr = jaclr * G . (deqP5.FFInverse[G] - D[FFInverse[G],P5]) // ReplaceAll[d -> 4-2\[Epsilon]] // stoy // Factor;


deqlr // Variables


(*Export["deq_y1.m", deqlr]*)


(* ::Section:: *)
(*compare: perfect agreement*)


num = numrep @ Variables @ deqmm


deqmm - deqlr // num // Column
