(* ::Package:: *)

(* ::Section:: *)
(*D-module*)


(* ::Text:: *)
(*GKZ vars: x[i]. Poly vars: zi*)


gkzform[poly_Plus,vars_List] := Module[{cr,crx,coeffs,gkzpoly},
	cr = CoefficientRules[poly, vars, "NegativeDegreeLexicographic"];
	crx = Table[cr[[i,1]] -> x[i], {i,Length@cr}];
	coeffs = Thread[(Range[Length@cr] // Map@x) -> cr[[All,2]]];
	(* The 'Reverse' needed because of 'Negative'DegLex*)
	gkzpoly = FromCoefficientRules[crx, Reverse@vars]; 
	
	{gkzpoly , coeffs}
]


(* ::Text:: *)
(*A-matrix for the case of one polynomial in the integrand*)


Amat[poly_Plus,vars_List] := Module[{togkz,powers},
	togkz = List @@ gkzform[poly, vars] [[1]];
	powers = CoefficientRules[# , vars] [[All,1,1]] &@ togkz;
	
	Join[{ConstantArray[1, Length@togkz]} , Transpose@powers]
]


(* ::Text:: *)
(*qs to rs. Use ns = {} if there is no nullspace*)


rvec[A_List,q_List,ns__List] := Module[{as,rs,rsol},
	If[Length@q != Length@A , Print["Dimension error"],
		as = Transpose@A;
		rs = Table[r@i , {i,Length@as}];
		rsol = Solve[Total[as * rs] == q , rs, Integers] /. C[_] -> 0 // Quiet;
	
		If[Length@rsol == 0, Print["No solution. The column vectors of A do not span Z."], 
			addns[ns, rsol[[1,All,2]]]
		]
	]
]


(* ::Text:: *)
(*Add nullspace to simplify r-vectors*)


addns[{}, rvec_] := rvec
addns[ns_, rvec_] := Prepend[ns // PadLeft[#, # // Dimensions // # + {0, 1}&]&, Prepend[rvec, 1]] // RowReduce // LatticeReduce // Cases[{1, __}] // Part[#, 1, 2;;]&


(* ::Text:: *)
(*To j[...] notation*)


dxtoj[vars_] := RightComposition[
	Collect[#, vars, $coeff]&,
	Expand,
	ReplaceAll[$coeff[h_] mon_. :> Times[h, mon // Exponent[#, vars]& // Apply@j]],
	Identity
];


(* ::Text:: *)
(*Order by complexity*)


order = SortBy[#, {
	# // Apply[List] // Multiply[-1] // Total,
	# // Apply[List] // Reverse,
	{}
}&]&;


(* ::Text:: *)
(*Maps d_xi \[Bullet] j[...ni...] \[RightArrow] j[...ni+1...]. Assumes no x-dependent prefactors*)


der[js_,dir_Integer] := Module[{len,m},
	len = js // filter[_j] // First // Length;
	If[dir > len, Print["dir > len"],
		m = UnitVector[len, dir];
		js /. j[n__] :> j @@ ({n} + m)
	]
]


(* ::Text:: *)
(*Factoring out matrices for C1' and C2'*)
(*TO DO: Change name*)


coeffarr[dir_,list_] := CoefficientArrays[der[std,dir], list][[2]] // Normal;


(* ::Text:: *)
(*exponents*)


btoed[expr__] := Module[{bs},
	bs = expr // Variables // Map@sym2ind // filter[_b] // Map@ind2sym;
	bs = Join[{
		First @ bs -> \[Epsilon]}, 
		Thread[Rest @ bs -> - \[Epsilon] \[Delta]]
	];
	expr /. bs
]


(* ::Text:: *)
(*Given e.g. dxs = Range[i, j] // Map@dx // Map@ind2sym*)


niceop[op_,dxs_] := op // btoed // dxtoj[dxs] // jsimp


(* ::Text:: *)
(*Import Macaulay matrix data. *)
(*OBS: b1 \[RightArrow] eps and bi \[RightArrow] - eps * del*)


mmimport[mmdata_String] := mmdata // RightComposition[
	Import,
	StringReplace[{"(1)*<<" -> "j[",">>"     -> "]"}],
	ToExpression,
	btoed,
	Factor
]


(* ::Text:: *)
(*gauge transformation p = (d_i \[Bullet] G + G \[Bullet] p_std) \[Bullet] Ginv.*)
(*e: coefficient matrix*)
(*g: from basis change algorithm*)


gauge[dir_, e_List, g_List] := Module[{P},

	P = (D[g,dir] + g.p[dir]) . Inverse[g] // Factor;
	
	e.P.Inverse[e] // RightComposition[
	FunctionExpand,
	# + O[\[Delta]]&,
	Normal,
	Factor, 
	Identity
	]
]


(* ::Section:: *)
(*LP*)


(* ::Text:: *)
(*Expand to lowest non-trivial order in \[Delta]*)


series[expr_, var_] := Module[{n = 0, res}, 
	res = expr;
	While[expr =!= 0 && 
		Normal[res = Series[expr, {var, 0, n}]] === 0,
		n++
	]; 
res // Normal // FunctionExpand]


(* ::Text:: *)
(*Lee-Pomeranski prefactor (L = 1 default)*)


LPfac[\[Nu]_List][d0_Integer:4,L_Integer:1] := Gamma[d0/2-\[Epsilon]] / (Gamma[(L+1)(d0/2-\[Epsilon])-Total[\[Nu]+\[Epsilon]*\[Delta]]] * Times @@ Gamma @ (\[Nu]+\[Epsilon]*\[Delta])) // FunctionExpand


(* ::Text:: *)
(*Scaling zi \[RightArrow] zi/s*)


kinfac[\[Nu]_List,s_][d0_Integer:4] := s^(d0/2-\[Epsilon]-Total[\[Nu]+\[Epsilon]*\[Delta]])


(* ::Text:: *)
(*Combined prefactor*)


fac[\[Nu]_List,s_][d0_Integer:4,L_Integer:1] := LPfac[\[Nu]][d0,L] * kinfac[\[Nu],s][d0]
